==> mule-metadata-model-api-1.6.0-sources.jar: org/mule/metadata/api/builder/IntersectionTypeBuilder.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.metadata.api.builder;

import org.mule.metadata.api.annotation.TypeAnnotation;
import org.mule.metadata.api.annotation.TypeIdAnnotation;
import org.mule.metadata.api.model.IntersectionType;
import org.mule.metadata.api.model.MetadataFormat;
import org.mule.metadata.api.model.MetadataType;
import org.mule.metadata.api.model.impl.DefaultIntersectionType;
import org.mule.metadata.internal.utils.LazyValue;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

public class IntersectionTypeBuilder extends AbstractBuilder<IntersectionType>
    implements TypeBuilder<IntersectionType>, WithAnnotation<IntersectionTypeBuilder> {

  private List<TypeBuilder<?>> types = new LinkedList<>();

  private final LazyValue<IntersectionType> result = new LazyValue<>();

  protected IntersectionTypeBuilder(MetadataFormat format) {
    super(format);
  }

  public BaseTypeBuilder of() {
    final BaseTypeBuilder builder = new BaseTypeBuilder(format);
    this.types.add(builder);
    return builder;
  }

  public IntersectionTypeBuilder of(TypeBuilder<?> of) {
    this.types.add(of);
    return this;
  }

  public IntersectionTypeBuilder id(String typeIdentifier) {
    return with(new TypeIdAnnotation(typeIdentifier));
  }

  public IntersectionTypeBuilder with(TypeAnnotation extension) {
    this.addExtension(extension);
    return this;
  }


  @Override
  public IntersectionType build() {
    if (types.isEmpty()) {
      throw new RuntimeException("IntersectionType needs at least one type");
    }
    final ArrayList<MetadataType> types = new ArrayList<>();
    // This is to avoid stack overflow we create the object with no fields
    boolean needsInit = !result.isDefined();
    final IntersectionType intersectionType = result.get(() -> new DefaultIntersectionType(types, format, annotations));
    if (needsInit) {
      final List<MetadataType> createdTypes = this.types
          .stream()
          .map((builder) -> builder.build())
          .collect(Collectors.toList());
      types.addAll(createdTypes);
    }
    return intersectionType;
  }
}

==> mule-metadata-model-api-1.11.0-20250813.184536-8-sources.jar: org/mule/metadata/api/builder/IntersectionTypeBuilder.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.metadata.api.builder;

import org.mule.metadata.api.annotation.TypeAnnotation;
import org.mule.metadata.api.annotation.TypeIdAnnotation;
import org.mule.metadata.api.model.IntersectionType;
import org.mule.metadata.api.model.MetadataFormat;
import org.mule.metadata.api.model.MetadataType;
import org.mule.metadata.api.model.impl.DefaultIntersectionType;
import org.mule.metadata.internal.utils.LazyValue;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

public class IntersectionTypeBuilder extends AbstractBuilder<IntersectionType>
    implements TypeBuilder<IntersectionType>, WithAnnotation<IntersectionTypeBuilder> {

  private final List<TypeBuilder<?>> types = new LinkedList<>();

  private final LazyValue<IntersectionType> result = new LazyValue<>();

  protected IntersectionTypeBuilder(MetadataFormat format) {
    super(format);
  }

  public BaseTypeBuilder of() {
    final BaseTypeBuilder builder = new BaseTypeBuilder(format);
    this.types.add(builder);
    return builder;
  }

  public IntersectionTypeBuilder of(TypeBuilder<?> of) {
    this.types.add(of);
    return this;
  }

  public IntersectionTypeBuilder id(String typeIdentifier) {
    return with(new TypeIdAnnotation(typeIdentifier));
  }

  @Override
  public IntersectionTypeBuilder with(TypeAnnotation extension) {
    this.addExtension(extension);
    return this;
  }


  @Override
  public IntersectionType build() {
    if (types.isEmpty()) {
      throw new RuntimeException("IntersectionType needs at least one type");
    }
    final List<MetadataType> types = new ArrayList<>();
    // This is to avoid stack overflow we create the object with no fields
    boolean needsInit = !result.isDefined();
    final IntersectionType intersectionType = result.get(() -> new DefaultIntersectionType(types, format, annotations));
    if (needsInit) {
      final List<MetadataType> createdTypes = this.types
          .stream()
          .map((builder) -> builder.build())
          .collect(Collectors.toList());
      types.addAll(createdTypes);
    }
    return intersectionType;
  }
}
